#ifndef _LIB_X11_H_
#define _LIB_X11_H_

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/keysym.h>
#include <X11/extensions/XShm.h>

#ifdef HAVE_LIBXPM
#  include <X11/xpm.h>
#endif

#include "lib_image.h"
#include "lib_font.h"


#define X11_MAX_COLORS   (SIZEOF_PALETTE / 3)


struct WINDOW_DATA
{
   Window                 window;
   XSetWindowAttributes   attr;
   long                   attr_mask;
   XGCValues              gc_values;
   long                   gc_values_mask;
   GC                     gc;
   char                  *window_name;
   char                  *icon_name;
   char                 **icon;
   XEvent                 event;
   long                   event_mask;
   XImage                *ximage;
   XShmSegmentInfo        shm_info;
   short                  use_mitshm;
   long                   shared_len;
   unchar                *shared_memory;
};

struct Color
{
   ushort red;
   ushort green;
   ushort blue;
};


extern Display        *x11_display;
extern int             x11_screen, x11_depth, x11_shm_completion;
extern XFontStruct    *x11_font_info;
extern Window          x11_win_root;
extern Colormap        x11_default_colormap, x11_private_colormap;
extern ulong           Pixel_color[X11_MAX_COLORS];
extern XVisualInfo     x11_visual_info;
extern XKeyboardState  x11_kbd_state;
extern short           x11_bpp;

extern void (*X11_callback_keyevent) (XKeyEvent *event);


#if defined (__STDC__) || defined (__cplusplus) || defined (NeedFunctionPrototypes)
#  define __PARMS(s) s
#else
#  define __PARMS(s) ()
#endif

extern int XShmGetEventBase __PARMS((Display *display));

short X11_init               __PARMS((void));
void  X11_close              __PARMS((void));
void  X11_handle_events      __PARMS((void));
short X11_set_color          __PARMS((ulong red, ulong green, ulong blue, short color_index));
short X11_set_colormap       __PARMS((palette_t *palette, short ncolors));
void  X11_colormap_fade      __PARMS((short direction));
short X11_window_open        __PARMS((struct WINDOW_DATA *win_data, short width, short height));
void  X11_window_close       __PARMS((struct WINDOW_DATA *win_data));
short X11_window_init_mitshm __PARMS((struct WINDOW_DATA *win_data));
void  X11_copyscreen         __PARMS((struct WINDOW_DATA *win_data));
void  X11_draw_line          __PARMS((struct WINDOW_DATA *win_data, short x1, short y1, short x2, short y2, short color_index));
void  X11_draw_rectangle     __PARMS((struct WINDOW_DATA *win_data, short x, short y, short width, short height, short color_index));
short X11_save_snapshot      __PARMS((Window win));

#undef __PARMS
#endif /* _LIB_X11_H_ */
