/***
 * Copyright (C) 1997,98 -- Miroslav Zagorac  <zaga@fly.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <sys/param.h>

#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#else
#  include <varargs.h>
#endif

#include "lib_atexit.h"
#include "lib_debug.h"
#include "lib_options.h"

#ifdef USE_LIBMEM_FUNC
#  include "lib_memory.h"
#endif

#ifdef USE_LIBLOG_FUNC
#  include "lib_log.h"
#endif


#ifdef DEBUG

static FILE  *debugfile_fd = NULL;
static char  *debug_buffer = NULL;
static long   buffer_size  = 0L;
static short  indent_level = 0, debug_tmp_flag = 0;
/***
 * debug_flag oznacava stanje debug funkcija:
 *
 * (debug_flag < 0)   ->  Debugiranje se obavlja, ali se svi logovi spremaju u memoriju.
 * (debug_flag == 0)  ->  Nema debugiranja.
 * (debug_flag > 0)   ->  Debugiranje se logira u datoteku.
 */
static short debug_flag = -1;

int __errno__;


#ifdef ANSI_FUNC

void debug_open (char *filename)
#else

void debug_open (filename)
   char *filename;
#endif
{
   char debug_filename[MAXPATHLEN];

   if (debugfile_fd)
      debug_close ();

   if (*filename)
      (void) strcpy (debug_filename, filename);
   else
#ifdef USE_LIBLOG_FUNC
      get_logname (debug_filename, "debug");
#else
   {
      char *tmpdir = getenv ("TMPDIR");

      (void) sprintf (debug_filename, "%s/debug.log", tmpdir ? tmpdir : "/tmp");
   }
#endif

   if (! (debugfile_fd = fopen (debug_filename, "wt")))
   {
      if (use_perror)
         perror (debug_filename);

      Exit (EX_CANTCREAT);
   }

   debug_flush ();

#ifdef USE_LIBLOG_FUNC
   log_write (LT_NORMAL, "Debug file is: %s\n", debug_filename);
#endif
}


#ifdef ANSI_FUNC

void debug_close (void)
#else

void debug_close ()
#endif
{
   if (debug_flag)
   {
      debug_print ("*** DEBUG: Closing all functions...\n");

      debug_flag = 1;

      /***
       * Treba natjerati debug da zatvori sve funkcije...
       */
      while (indent_level > 0)
	 debug_endfunc

      debug_flush ();
      (void) fclose (debugfile_fd);
   }

   debugfile_fd = NULL;
   debug_flag   = debug_tmp_flag = 0;
}


#ifdef ANSI_FUNC

void debug_on (void)
#else

void debug_on ()
#endif
{
   debug_flag = debug_tmp_flag;
}


#ifdef ANSI_FUNC

void debug_off (void)
#else

void debug_off ()
#endif
{
   debug_tmp_flag = debug_flag;
   debug_flag     = 0;
}


#ifdef ANSI_FUNC

void debug_flush (void)
#else

void debug_flush ()
#endif
{
   if (debug_buffer && debugfile_fd)
   {
      /***
       * Evo jedan mali trik ovdje obznanjen za svagda:
       *   Ako se funkcijama *printf() zeli ispisati nesto slicno ovome: %s%d
       *   tj. ako tekst koji se ispisuje u sebi ima neke kontrolne znakove za *printf()
       *   funkcije, onda se to mora ovako raditi:
       *
       *      printf (file_fd, "%s", text);
       *
       *   inace ce se printf zbuniti, te tako prouzrokovati mnoge core i ostalo...
       */
      (void) fprintf (debugfile_fd, "%s", debug_buffer);
      (void) fflush (debugfile_fd);

      free (debug_buffer);

      debug_buffer = NULL;
      buffer_size  = 0L;
   }

   debug_flag = debug_tmp_flag = 1;
}


#ifdef ANSI_FUNC

void debug_addtext (char *text)
#else

void debug_addtext (text)
   char *text;
#endif
{
   if (debug_flag > 0)
   {
      /***
       * Debug datoteka je otvorena (i debugiranje dozvoljeno), i u nju se upisuje tekst.
       */
      (void) fprintf (debugfile_fd, "%s", text);
      (void) fflush (debugfile_fd);
   }
   else if (debug_flag < 0)
   {
      /***
       * Debugiranje je zabranjeno i/ili debug datoteka jos nije otvorena.
       * Sav tekst se sprema u memoriju.
       */
      debug_off ();

      if (! debug_buffer)
      {
         debug_buffer  = (char *) MEMalloc (buffer_size = DEBUG_BLOCKSIZE);
         *debug_buffer = '\0';
      }
      else if (buffer_size < (strlen (debug_buffer) + strlen (text) + 4))
      {
         char *new_buffer = (char *) MEMrealloc (debug_buffer, buffer_size += DEBUG_BLOCKSIZE);

         if (new_buffer)
         {
            debug_buffer = new_buffer;

            (void) sprintf (debug_buffer + strlen (debug_buffer), ">>> Reallocating debug buffer... size is %ld bytes!\n", buffer_size);
         }
         else
         {
            /***
             * Oh, shit!
             */
            debug_on ();

            return;
         }
      }

      (void) strcat (debug_buffer, text);

      debug_on ();
   }
}


#ifdef HAVE_STDARG_H

void debug_print (char *format, ...)
#else

void debug_print (va_alist)
   va_dcl
#endif
{
   va_list ap;
   char    tmp_buffer[BUFSIZ], empty_space[BUFSIZ], debug_text[BUFSIZ], start_char;

   if (debug_flag)
   {
#ifndef HAVE_STDARG_H
      char *format;

      va_start (ap);

      format = va_arg (ap, char *);
#else
      va_start (ap, format);
#endif

      (void) vsprintf (tmp_buffer, format, ap);
      va_end (ap);

      if (strchr ("{}", (start_char = *tmp_buffer)))
         (void) memmove (tmp_buffer, tmp_buffer + 1, strlen (tmp_buffer));

      (void) memset (empty_space, ' ', BUFSIZ);

      switch (start_char)
      {
         case '{' : /***
                     * Oznaka za pocetak funkcije.
                     */
                    (void) sprintf (debug_text, "%.*s%s\n%.*s{\n", indent_level * DEBUG_INDENT_WIDTH, empty_space, tmp_buffer, indent_level * DEBUG_INDENT_WIDTH, empty_space);

                    indent_level++;

                    break;
 
         case '}' : /***
                     * Oznaka za kraj funkcije.
                     */
                    if (--indent_level >= 0)
                       (void) sprintf (debug_text, "%.*s}%s\n", indent_level * DEBUG_INDENT_WIDTH, empty_space, tmp_buffer);
                    else
#ifdef USE_LIBLOG_FUNC
                       BUG ("error in debugger!");
#else
                       (void) sprintf (debug_text, "*** BUG: error in debugger!\n");
#endif
 
                       break;

         case '' : /***
                     * String koji se upisuje u debug buffer se ne indentira, tj. upisuje se
                     * u originalnom obliku bez pocetnih praznina.
                     */
                    (void) strcpy (debug_text, tmp_buffer + 1);

                    break;

         default  : (void) sprintf (debug_text, "%.*s%s", indent_level * DEBUG_INDENT_WIDTH, empty_space, tmp_buffer);
      }

      debug_addtext (debug_text);
   }
}

#endif /* DEBUG */


#ifdef ANSI_FUNC

void perror_exit (char *msg, short errnum)
#else
   
void perror_exit (msg, errnum)
   char *msg;
   short errnum;
#endif
{
   if (use_perror)
      (void) fprintf (stderr, "%s%s%s\n", msg, (errnum < 0) ? ": " : "", (errnum < 0) ? strerror (-errnum) : "");

   Exit (errnum);
}


#ifdef ANSI_FUNC

time_t timeval_diff (struct timeval *tv_1, struct timeval *tv_2)
#else
   
time_t timeval_diff (tv_1, tv_2)
   struct timeval *tv_1;
   struct timeval *tv_2;
#endif
{
   time_t retval;

   debug (("{%s(0x%08lx, 0x%08lx)", __FUNCTION__, (long) tv_1, (long) tv_2))

   retval = 1000000L * (tv_1->tv_sec - tv_2->tv_sec) + (tv_1->tv_usec - tv_2->tv_usec);

   debug_return ("} == %ld", retval)
}
