/***
 * Copyright (C) 1997,98 -- Miroslav Zagorac  <zaga@fly.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <ctype.h>
#include <string.h>
#include <sys/types.h>

#include "lib_debug.h"
#include "lib_string.h"


#ifdef ANSI_FUNC

char *strlwr (char *string)
#else

char *strlwr (string)
   char *string;
#endif
{
   if (string)
   {
      char *ptr;

      for (ptr = string; *ptr; ptr++)
         *ptr = tolower ((int) *ptr);
   }

   return (string);
}


#ifdef ANSI_FUNC

char *strupr (char *string)
#else

char *strupr (string)
   char *string;
#endif
{
   if (string)
   {
      char *ptr;

      for (ptr = string; *ptr; ptr++)
         *ptr = toupper ((int) *ptr);
   }

   return (string);
}


#ifdef ANSI_FUNC

short fls (ulong value)
#else
   
short fls (value)
   ulong value;
#endif
/***
 * fls() -- find last bit set in a word (opposite of ffs()).
 */
{
   short retval;

   debug (("{%s(0x%08lx)", __FUNCTION__, value))

   for (retval = sizeof (ulong) * 8; retval > 0; retval--, value <<= 1)
      if (value & 0x80000000UL)
         break;

   debug (("value = 0x%08lx\n", value))
   debug_return ("} == %d", retval)
}


#ifdef WORDS_BIGENDIAN

#ifdef ANSI_FUNC

ushort swapw (ushort *value)
#else

ushort swapw (value)
   ushort *value;
#endif
{
   ushort new_value = ((*value >> 8) & 0x00ff) | ((*value << 8) & 0xff00);

   return (*value = new_value);
}


#ifdef ANSI_FUNC

ulong swapl (ulong *value)
#else

ulong swapl (value)
   ulong *value;
#endif
{
   ulong new_value = ((*value >> 24) & 0x000000ffUL) | ((*value >> 8) & 0x0000ff00UL) | ((*value << 8) & 0x00ff0000UL) | ((*value << 24) & 0xff000000UL);

   return (*value = new_value);
}

#endif /* WORDS_BIGENDIAN */


#ifndef HAVE_STRSIGNAL

#ifndef ANSI_FUNC
char *strsignal (int signum)

#else
char *strsignal (signum)
   int signum;
#endif
{
   return ("");
}

#endif /* HAVE_STRSIGNAL */


#ifndef HAVE_STRERROR

#ifndef ANSI_FUNC
char *strerror (int errnum)

#else
char *strerror (errnum)
   int errnum;
#endif
{
   static char retbuf[64];

   if ((error_nr >= 0) && (error_nr < sys_nerr) && sys_errlist[error_nr])
      return (sys_errlist[error_nr]);
   else
   {
      (void) sprintf (retbuf, "Error #%d", error_nr);

      return (retbuf);
   }
}

#endif /* HAVE_STRERROR */


#ifndef HAVE_MEMMOVE

#ifdef ANSI_FUNC

void *memmove (void *dest, void *src, size_t n)
#else

void *memmove (dest, src, n)
   void *dest;
   void *src;
   size_t n;
#endif
{
   if ((n >= 0) && dest && src)
   {
#ifdef HAVE_BCOPY
      (void) bcopy (src, dest, n);
#else
      char *dest_ptr = dest, *src_ptr = src;

      for ( ; n--; dest_ptr++, src_ptr++)
         *dest_ptr = *src_ptr;
#endif
   }

   return (dest);
}

#endif /* HAVE_MEMMOVE */
